
---------------------------------------------------------------------------------
-- How much memory is in the Cache?
---------------------------------------------------------------------------------
-- What is the cache?
SELECT  * 
FROM sys.dm_os_memory_cache_counters 
order by single_pages_kb + multi_pages_kb DESC

-- How big is the cache?
SELECT sum(single_pages_kb + multi_pages_kb) AS Cache_KB
	, sum(single_pages_kb + multi_pages_kb) / 1024 AS Cache_MB
FROM sys.dm_os_memory_cache_counters  

-- How big is each piece of the cache
SELECT [name], [type], sum(single_pages_kb + multi_pages_kb) AS Cache_KB
	, sum(single_pages_kb + multi_pages_kb) / 1024 AS Cache_MB
FROM sys.dm_os_memory_cache_counters  
GROUP BY [name], [type]
ORDER BY sum(single_pages_kb + multi_pages_kb) DESC


SELECT
	LEFT([name], 20) AS [Name],
	LEFT([type], 20) as [Type],
	single_pages_kb,
	single_pages_kb / 1024 AS cache_size_MB,
	entries_count,
	avg_size_kb = single_pages_kb / entries_count
FROM
	sys.dm_os_memory_cache_counters
WHERE 
	[type] in ('CACHESTORE_SQLCP', 'CACHESTORE_OBJCP', 'CACHESTORE_PHDR')
AND
	entries_count <> 0
ORDER BY
	single_pages_kb DESC


-- RUN AFTER 

-- What's in the cache?
select  TOP 10 [sql].[text], p.size_in_bytes, p.usecounts
from sys.dm_exec_cached_plans p
cross apply sys.dm_exec_sql_text (p.plan_handle) sql
order by p.usecounts DESC
GO



/*

     Single_Pages_KB      Single_Pages_MB
-------------------- --------------------
              671848                  656

(1 row(s) affected)

Name                 Type                      single_pages_kb      Single_Pages_MB        entries_count
-------------------- -------------------- -------------------- -------------------- --------------------
SQL Plans            CACHESTORE_SQLCP                   657144                  641                 9520
Object Plans         CACHESTORE_OBJCP                    10128                    9                  104
Bound Trees          CACHESTORE_PHDR                       776                    0                   15
Extended Stored Proc CACHESTORE_XPROC                       24                    0                    1

(4 row(s) affected)


*/