-----------------------------------------------------------------------------------------------------------------
-- Setup  & Start Profiler & Display Actual Execution Plan
-----------------------------------------------------------------------------------------------------------------
use AdventureWorks
GO
ALTER DATABASE [AdventureWorks] SET PARAMETERIZATION SIMPLE 
GO
/****** Object:  StoredProcedure [dbo].[ParameterSniffing]    Script Date: 05/08/2007 20:25:17 ******/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffing]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffing]


/****** Object:  Index [IX_SalesOrderHeader_RegionID]    Script Date: 05/08/2007 20:09:04 ******/
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[Sales].[SalesOrderHeader]') AND name = N'IX_SalesOrderHeader_RegionID')
	DROP INDEX [IX_SalesOrderHeader_RegionID] ON [Sales].[SalesOrderHeader] WITH ( ONLINE = OFF )

IF  EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'RegionID' AND TABLE_NAME = 'SalesOrderHeader')
	ALTER TABLE Sales.SalesOrderHeader DROP COLUMN  RegionID
GO



IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'RegionID' AND TABLE_NAME = 'SalesOrderHeader')
	ALTER TABLE Sales.SalesOrderHeader
	  ADD RegionID INT NULL
GO
UPDATE Sales.SalesOrderHeader
SET		RegionID = 1

UPDATE Sales.SalesOrderHeader
SET		RegionID = 2
WHERE	SalesOrderID IN (SELECT TOP 5 SalesOrderID FROM Sales.SalesOrderHeader ORDER BY NEWID() )

CREATE INDEX IX_SalesOrderHeader_RegionID ON Sales.SalesOrderHeader(RegionID)
GO

---------------------------------------------------------------------------------------------------------
-- Start Profiler & How many query plans?
---------------------------------------------------------------------------------------------------------
DBCC FREEPROCCACHE
GO
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = 2
GO
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = 1
GO

select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO

-------------------------- Add another predicate -----------------------------------------------
DBCC FREEPROCCACHE
GO
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = 2 AND SalesOrderID = 22000
GO
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = 1 AND SalesOrderID = 22001
GO

select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO



---------------------------------------------------------------------------------------------------------
-- Demo stored procedure parameterization
---------------------------------------------------------------------------------------------------------
/****** Object:  StoredProcedure [dbo].[ParameterSniffing]    Script Date: 05/08/2007 20:25:17 ******/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffing]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffing]

GO
CREATE PROC	ParameterSniffing
	@RegionID INT
AS
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = @RegionID
GO

-- Both Run Clean ----------------------------------------------
-- Show Query Plan
DBCC FREEPROCCACHE
EXEC ParameterSniffing 1
GO

DBCC FREEPROCCACHE
EXEC ParameterSniffing 2
GO


-- 1 then 2 ----------------------------------------------------
-- Index Usage
DBCC FREEPROCCACHE
GO
EXEC ParameterSniffing 1
GO
EXEC ParameterSniffing 2
GO


select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO


-- 2 then 1 -----------------------------------------------------
-- INclude the actual query plan
DBCC FREEPROCCACHE
GO
EXEC ParameterSniffing 2
GO
EXEC ParameterSniffing 1 
GO


-- Demo on the client side --------------------------------------
--DBCC FREEPROCCACHE
--GO
--
--select  [sql].[text], stats.execution_count, p.size_in_bytes
--from sys.dm_exec_cached_plans p
--outer apply sys.dm_exec_sql_text (p.plan_handle) sql
--join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
--GO

