use AdventureWorks
GO
---------------------------------------------------------------------------------------------------------
-- Using WITH RECOMPILE at Execution time
---------------------------------------------------------------------------------------------------------
DBCC FREEPROCCACHE
GO
EXEC ParameterSniffing 2 WITH RECOMPILE
GO
EXEC ParameterSniffing 1 WITH RECOMPILE  -- no plan saved
GO
select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO

---------------------------------------------------------------------------------------------------------
-- Using WITH RECOMPILE at Compile Time
---------------------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffingRecompile]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffingRecompile]
GO
CREATE PROC	ParameterSniffingRecompile
	( @RegionID INT ) 
        WITH RECOMPILE -- This is different
AS
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = @RegionID 
GO


DBCC FREEPROCCACHE
GO
EXEC ParameterSniffingRecompile 2
GO
EXEC ParameterSniffingRecompile 1
GO
select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO



---------------------------------------------------------------------------------------------------------
-- Using OPTION (RECOMPILE)
---------------------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffingRecompile]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffingRecompile]
GO
CREATE PROC	ParameterSniffingRecompile
	@RegionID INT  ----- This is different
AS
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = @RegionID 
OPTION(RECOMPILE)  ----- This is different

--SELECT COUNT(*) FROM Sales.SalesOrderHeader WHERE SalesOrderID = @SalesOrderID
GO

DBCC FREEPROCCACHE
GO
EXEC ParameterSniffingRecompile 2
GO
EXEC ParameterSniffingRecompile 1
GO
select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO


---------------------------------------------------------------------------------------------------------
-- Using OPTION ( OPTIMIZE FOR() )
---------------------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffingOptimizedFor]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffingOptimizedFor]
GO
CREATE PROC	[ParameterSniffingOptimizedFor]
	@RegionID INT  
AS
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = @RegionID 
OPTION(OPTIMIZE FOR (@RegionID = 1))  ----- This is different
GO

DBCC FREEPROCCACHE
GO
EXEC [ParameterSniffingOptimizedFor] 2
GO
EXEC [ParameterSniffingOptimizedFor] 1
GO
select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO



---------------------------------------------------------------------------------------------------------
-- Replace Parameters with Local Variable
---------------------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffingLocalVariable]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffingLocalVariable]
GO
CREATE PROC	[ParameterSniffingLocalVariable]
	@RegionID INT
AS
DECLARE @LocalRegionID INT
SELECT	@LocalRegionID = @RegionID

SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = @LocalRegionID 
GO

DBCC FREEPROCCACHE
GO
EXEC [ParameterSniffingLocalVariable] 2
GO
EXEC [ParameterSniffingLocalVariable] 1
GO

select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO



---------------------------------------------------------------------------------------------------------
-- Resetting a parameter
---------------------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParameterSniffingRewrite]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[ParameterSniffingRewrite]
GO
CREATE PROC	[ParameterSniffingRewrite]
	@RegionID INT
AS
SELECT	@RegionID = 1

SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	RegionID = @RegionID 
GO

DBCC FREEPROCCACHE
GO
EXEC [ParameterSniffingRewrite] 2
GO

select  [sql].[text], stats.execution_count, p.size_in_bytes
from sys.dm_exec_cached_plans p
outer apply sys.dm_exec_sql_text (p.plan_handle) sql
join sys.dm_exec_query_stats stats ON stats.plan_handle = p.plan_handle
GO
